import hashlib

class HASH:
    def __init__(self,data:str) -> None:
        self.data = str(data)

    def encode(self):
        return hashlib.sha512(self.data.encode('utf-8')).hexdigest()
        





from HadiDB.paths import Path
from HadiDB.auth import Auth
from HadiDB.schema import arrangeData
from HadiDB.engine import FileHandler
from HadiDB.hash import HASH
from HadiDB.getbyID import GetByID
from HadiDB.haditechnique import HadiTechnique
import os
import itertools


class Update:
    def __init__(self,
                username:str,
                password:str,
                database:str,
                collection:str,
                objectid:int,
                data:dict
                 ) -> None:
        self.__id = objectid
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
        self.__message = "DB something Wrong"

        self.__data = data
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path
        self.__manage_File = Path(self.__database,self.__collection).manage_File
        self.__manage_json_file_data = FileHandler(self.__manage_File).read() 
        



    def __is_collection_exist(self):
        if self.__manage_json_file_data.get('collection') == []:
            return 404
        elif self.__manage_json_file_data.get('collection')[0].get(self.__collection) == None:
            return 404
        else:
            return 200



    def update(self):
        if Auth(self.__username,self.__password)._Authenticate() == None:
            return {
                "status":401,
                "message":"Wrong Username Password"
            }

        if self.__is_collection_exist() == 200:
            
            
            self.__schema = self.__manage_json_file_data.get('collection')[0].get(self.__collection)[0]

            validate_schema = self.__data.keys() == self.__schema.keys()

            if validate_schema == True:
                
                arrange_data = (arrangeData(self.__schema,self.__data))
                self.working_file = self.__manage_json_file_data.get('working_file')[0].get(self.__collection)[0]
                fileName = HadiTechnique(self.__id).map_number_to_range()
                
                self.actual_data_file_path = os.path.join(self.__db_full_path,f"{HASH(fileName).encode()}.json")
                
                self.get_actual_data = FileHandler(self.actual_data_file_path).read()
                

                if self.get_actual_data['data'] == []:
                    return {
                        "status":404,
                        "message":"collection is Empty"
                    }

                
                if self.get_actual_data['data'][0].get(str(self.__id)) == None:
                    return {
                        "status":404,
                        "message":"ID not Exist"
                    }

                

                
                if self.__update_unique_values()['status'] == 200:
                    arrange_data['id'] = self.__id
                    self.get_actual_data['data'][0].get(f"{self.__id}").update(arrange_data)
                    
                    FileHandler(self.actual_data_file_path).write(self.get_actual_data)
                    
                    return {
                        "status":200,
                        "message":"Data Update successfully",
                        "data":arrange_data
                    }

                else:
                    return {
                        "status":409,
                        "message": self.__message
                    }
            else:
                return {
                    "status":404,
                    "message":"Invalid Schema missmatch key"
                }
        else:
            return {
                "status":404,
                "message":"collection not found"
            }


    def __update_unique_values(self):
        self.__cache_data = []
        self.__count = 0
        for key,value in self.__schema.items():
            if value == "Unique":
                self.__count = self.__count + 1
                __key = HASH(key).encode()
                self.folder_path = os.path.join(self.__db_full_path,__key)
                
                files = [file[:-5] for file in os.listdir(self.folder_path) if file.endswith('.json')]
                
                get_actual_key_value = self.get_actual_data['data'][0].get(str(self.__id))[key]
                
                for filename in itertools.islice(files,len(files)):

            
                    
                    self.filepath_unique = os.path.join(self.__db_full_path,__key,f"{filename}.json")
                    self.unique_file_data = FileHandler(self.filepath_unique).read()

                
                    if bool(self.unique_file_data['data'][0].get(self.__data.get(key,None))) == True:
                        self.__message = f" {key} Already Available"
                        return {
                            "status":409
                        }


                    try:

                        del self.unique_file_data['data'][0][get_actual_key_value]
                        self.unique_file_data['data'][0][self.__data.get(key)] = self.__id
                        self.__cache_data.append({
                                
                                    "filepath":self.filepath_unique,
                                    "data":self.unique_file_data
                                
                            })
                    except:
                        
                        pass


            elif value == "Text":
                self.__count = self.__count + 1
                __key = HASH(key).encode()

                self.folder_path = os.path.join(self.__db_full_path,__key)
                
                files = [file[:-5] for file in os.listdir(self.folder_path) if file.endswith('.json')]
                get_actual_key_value = self.get_actual_data['data'][0].get(str(self.__id))[key]
                
                for filename in itertools.islice(files,len(files)):

                    self.filepath_text = os.path.join(self.__db_full_path,__key,f"{filename}.json")
                    
                    self.no_unique_file_data = FileHandler(self.filepath_text).read()

                    
                    
                    for item in itertools.islice(self.no_unique_file_data['data'] ,len(self.no_unique_file_data['data'])):
                        if item[0] == get_actual_key_value:
                            index = self.no_unique_file_data['data'].index(item)
                            
                            self.no_unique_file_data['data'].pop(int(index))
                            
                    
                            self.no_unique_file_data['data'].append([self.__data[key],self.__id])
                

                            self.__cache_data.append({
                                    
                                        "filepath":self.filepath_text,
                                        "data":self.no_unique_file_data
                                    
                                })
                        else:
                            pass

                        
        if len(self.count_unique_vlaues()) == len(self.__cache_data):
            for i in self.__cache_data:
                FileHandler(i['filepath']).write(i['data'])

        else:
            return {"status":404}


        return {
            "status":200
        }


    def count_unique_vlaues(self):
        self.total_keys = []
        for value in self.__schema.values():
            if value == "Unique" or value == "Text":
                self.total_keys.append(value)
        return self.total_keys


from HadiDB.paths import Path
from HadiDB.engine import FileHandler


class Config:

    def __init__(self,database:str=None,collection:str=None) -> None:
        self.collection = collection
        self.__database = database

    def getallcollection(self):
        collection = []
        self.manage_file = Path(self.__database).manage_File
        all_collection = FileHandler(self.manage_file).read()['collection'][0]
        for key,value in all_collection.items():
            collection.append(key)
        return collection
    
    def getalldatabase(self):
        database = []
        self.manage_File_bin = Path().manage_File_bin
        all_database = FileHandler(self.manage_File_bin).read()['database']
        for key in all_database:
            database.append(key)
        return database

    def getschema(self):
        self.manage_file = Path(self.__database).manage_File
        return FileHandler(self.manage_file).read()['collection'][0].get(self.collection)
        
        



from HadiDB.paths import Path
from HadiDB.hash import HASH
from HadiDB.engine import FileHandler
import shutil
import os

class DeleteDatabase:
    def __init__(self,database:str) -> None:
        self.database = database
        self.__database = database

    def delete(self):
        try:

            database_path = Path(self.database).manage_path_File
            manage = Path(self.database).manage_File_bin
            shutil.rmtree(database_path)
            read_manage = FileHandler(manage).read()
            remove_db = read_manage['database']
            remove_db.remove(self.database)
            FileHandler(manage).write(read_manage)
            return {
                "status":200,
                "message":f"{self.__database} Database delete successful"
            }
        except:
            return {
                "status":404,
                "message":f"{self.__database} database not found"
            }



from HadiDB.paths import Path
from HadiDB.engine import FileHandler


class CountByAll:
    def __init__(self,database,collection) -> None:
        self.__database = database
        self.__collection = collection
        self.manage_File = Path(self.__database,self.__collection).manage_File
        self.manageFile = FileHandler(self.manage_File).read()



    def get(self):
        
        working_file = self.manageFile['working_file'][0].get(self.__collection)
        delete_count = self.manageFile['delete_count'][0].get(self.__collection)
        
        if working_file != None and delete_count != None:
            total_count = int(working_file[0]['ID']) - int(delete_count)
            return {
                "status":200,
                "total_count":total_count
            }
            



from HadiDB.hash import HASH
from HadiDB.paths import Path
from HadiDB.auth import Auth
from HadiDB.engine import FileHandler
from HadiDB.schema import Schema

class Database:
    def __init__(self,
                 username:str,
                 password:str,
                 database:str,
                 collection:str,
                 schema:dict
                 ) -> None:
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
        self.__schema = schema

    def create(self):      

        if type(self.__schema).__name__ != "dict":
            return {
                "message":"pass only dict"
            }

        self.auth_file = Path().auth_File
        self.__manage_File_bin = Path(self.__database).manage_File_bin
        if Auth(self.__username,self.__password)._Authenticate():
            manage_json_bin = FileHandler(self.__manage_File_bin).read()
            if self.__database in manage_json_bin['database']:
                pass
            else:
                manage_json_bin['database'].append(self.__database)
                FileHandler(self.__manage_File_bin).write(manage_json_bin)

            return Schema(self.__schema,self.__database,self.__collection).validate()

            
        else:
            return { "status":401,"message":"Wrong username and password" }
        
    def jsonKeyExists(self,collection,list_of_dicts):
        for rng in range(0,len(list_of_dicts)):
            for key in (list_of_dicts[rng].keys()):
                if collection == key:
                    return True
                else:
                    return False


from HadiDB.haditechnique import HadiTechnique
from HadiDB.engine import FileHandler
from HadiDB.auth import Auth
import os
from HadiDB.paths import Path
from HadiDB.hash import HASH

class GetByID:
    def __init__(self,
                username:str,
                password:str,
                database:str,
                collection:str,
                
                 ) -> None:
        
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path

    def get(self,objectID):
        self.__id = objectID
        if Auth(self.__username,self.__password)._Authenticate() == None:
            return {
                "status":401,
                "message":"Wrong Username Password"
            }
        try:
            filename =  HASH(HadiTechnique(self.__id).map_number_to_range()).encode()
            filepath = os.path.join(self.__db_full_path,f"{filename}.json")
            data = FileHandler(filepath).read()
            if data['data'][0].get(str(self.__id)) == None:
                return {
                    "status":404,
                    "message":"data not found"
                }

            return {
                    "status":200,
                    "message":"Data Found",
                    "data":data['data'][0].get(str(self.__id))   
                }
        except:
            return {
                    "status":404,
                    "message":"data not found"
                }
        


    def get_all_data_file(self):

        if Auth(self.__username,self.__password)._Authenticate() == None:
            return {
                "status":401,
                "message":"Wrong Username Password"
            }

        filename = HASH(HadiTechnique(self.__id).map_number_to_range()).encode()
        filepath = os.path.join(self.__db_full_path,f"{filename}.json")
        data = FileHandler(filepath).read()
        if data['data'][0].get(str(self.__id)) == None:
            return {
                "status":404,
                "message":"data not found"
            }
        return data




from HadiDB.paths import Path
from HadiDB.hash import HASH
from HadiDB.engine import FileHandler
import os
import shutil


class DeleteCollection:
    def __init__(self,database:str,collection:str) -> None:
        self.collection = collection
        self.database = database
        self.__collection = collection
        self.__database = database
    def delete(self):
        try:

            collection_path = Path(self.database,self.collection).db_full_path
            
            collection_path_manage = Path(self.database,self.collection).db_full_path_json
            read_manage = FileHandler(collection_path_manage).read()

            collection = read_manage.get('collection',None)
            working_file = read_manage.get('working_file',None)
            reference_value = read_manage.get('reference_value',None)
            delete_count = read_manage.get('delete_count',None)

            if collection != None and collection != []:
                collection[0].pop(self.__collection)
            if working_file != None and working_file != []:

                working_file[0].pop(self.__collection)
            if reference_value != None and reference_value != []:

                reference_value[0].pop(self.__collection)
            # if delete_count != None and delete_count != []:

            #     delete_count[0].pop(self.__collection)

            shutil.rmtree(collection_path)
            FileHandler(collection_path_manage).write(read_manage)
            return {
                    "status":200,
                    "message":f"{self.__collection} Collection delete successful"
                }
        except:
            return {
                    "status":404,
                    "message":f"{self.__collection} Collection Not Found"
                }
        






from HadiDB.paths import Path
import os
import itertools
from HadiDB.engine import FileHandler



class GetByKeys:
    def __init__(self,collection,database) -> None:
        self.__collection = collection
        self.__database = database
        self.db_full_path = Path(self.__collection,self.__database).db_full_path


    def get(self, data: dict):
        self.__data = data
        result = []
        
        files = [file for file in os.listdir(self.db_full_path) if file.endswith('.json')]

        for file in itertools.islice(files, len(files)):
            file_path = os.path.join(self.db_full_path, file)
            
            filedata = FileHandler(file_path).read()

            if filedata is not None:
                for record_id, record in itertools.islice(filedata.get('data', [{}])[0].items(),len(filedata.get('data', [{}])[0].items())):
                    if all(record.get(key) == value for key, value in itertools.islice(self.__data.items(),len(self.__data.items()))):
                        result.append(record)
                        
                    else:
                        pass
        if result != []:
            return {
                "status":200,
                "data": result
            }           
        else:
            return {
                "status":404,
                "message":"Data not found"
            }



from HadiDB.paths import Path
from HadiDB.engine import FileHandler
from HadiDB.operation import GetByID
from HadiDB.hash import HASH
import os
import itertools


class GetByKey:
    def __init__(self,
                username:str,
                password:str,
                database:str,
                collection:str,
                data:dict,
                 ) -> None:
        self.__data = data
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection

        
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path
        self.get_data = GetByID(self.__username,self.__password,self.__database,self.__collection)
        
        



    def get(self):
    
        values = []
        __key = HASH(list(self.__data.keys())[0]).encode()
        self.full_file_path = os.path.join(self.__db_full_path,__key)
        

        if os.path.exists(self.full_file_path) == False:
            return {
                "status":404,
                "message":"Key Not Found"
            }
    
        files = os.listdir(self.full_file_path)
        
        
        for file in itertools.islice(files, len(files)):
            
            filedata = FileHandler(
                os.path.join(self.full_file_path,file)
            ).read()
            try:
                
                for item in itertools.islice(filedata['data'], len(filedata['data'])):
                    if item[0] == list(self.__data.values())[0]:
                        values.append(self.get_data.get(item[1])['data'])
            except:
                
                for item in itertools.islice(filedata['data'], len(filedata['data'])):
                    result = item.get(list(self.__data.values())[0])
                    if result != None:
                        values.append(self.get_data.get(result))

            
        if values != []:
            return values
        else:
            return {
                "status":404,
                "message":"Data not Found"
            }
                


    def count(self):
        values = []
        __key = HASH(list(self.__data.keys())[0]).encode()
        self.full_file_path = os.path.join(self.__db_full_path,__key)
        

        if os.path.exists(self.full_file_path) == False:
            return {
                "status":404,
                "message":"Key Not Found"
            }
    
        files = os.listdir(self.full_file_path)
        
        
        for file in itertools.islice(files, len(files)):
            
            filedata = FileHandler(
                os.path.join(self.full_file_path,file)
            ).read()
            try:
                for item in itertools.islice(filedata['data'], len(filedata['data'])):
                    if item[0] == list(self.__data.values())[0]:
                        
                        values.append(self.get_data.get(item[1])['data'])
            except:
                for item in itertools.islice(filedata['data'], len(filedata['data'])):
                    result = bool(item.get(list(self.__data.values())[0]))
                    if result == True:
                        values.append(1)

            

        return len(values)
            
                
               


import os
from HadiDB.hash import HASH

class Path:
    def __init__(self,database:str=None,collection:str=None) -> None:
        

        ParentFolder = HASH(".Hadi").encode()
        authFile = HASH(".auth").encode()
        manageFile = HASH(".manage").encode()
        Bin = HASH(".bin").encode()
        

        self.bin_full_path = os.path.join(ParentFolder,Bin)
        self.auth_File = os.path.join(ParentFolder,Bin,f"{authFile}.json")
        self.manage_File_bin = os.path.join(ParentFolder,Bin,f"{manageFile}.json")
        if database != None:
            self.__database = HASH(database).encode()
            self.manage_File = os.path.join(ParentFolder,self.__database,f"{manageFile}.json")
            self.manage_path_File = os.path.join(ParentFolder,self.__database)
        if database != None and collection != None:
            self.__collection = HASH(collection).encode()
            self.__database = HASH(database).encode()
            self.db_full_path = os.path.join(ParentFolder,self.__database,self.__collection)
            self.db_full_path_json = os.path.join(ParentFolder,self.__database,f"{manageFile}.json")
    




class Write:
    def __init__(self,manage_file_data:dict,collection_name:str) -> None:

        self.manage_file_data = manage_file_data
        self.collection_name = collection_name

        print(self.manage_file_data['working_file'][0][self.collection_name])

    


import hashlib

class HadiTechnique:
    def __init__(self,objectID=None) -> None:
        if objectID == None:
            pass
        else:
            self.objectID = int(objectID)
        

    def hash_string(self,stringTohash):
        hasher = hashlib.sha256()
        hasher.update(stringTohash.encode('utf-8'))
        hashed_string = hasher.hexdigest()
        return hashed_string


    def map_number_to_range(self):
        range_start = ((self.objectID - 1) // 1000) * 1000 + 1
        range_end = ((self.objectID - 1) // 1000) * 1000 + 1000
        self.stringTohash = f"{range_start}_{range_end}"
        return self.stringTohash
        




from HadiDB.paths import Path
import os
from HadiDB.hash import HASH
from HadiDB.engine import FileHandler
from HadiDB.haditechnique import HadiTechnique




SCHEMARULE = [
    "Text",
    "Hash",
    "Unique",
    "Image"
]


EMPTY_FILE  = {
    "data": []
    }

EMPTY_DATABASE  = {
    "database": [],
    "collection": [],
    "schema": [],
    "working_file": [],
    "reference_value": [],
    "delete_count": []
    }

class Schema:
    def __init__(self,schema:dict,database,collection) -> None:
        self.__schema = schema
        self.__database = database
        self.__collection = collection
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path
        self.__manage_File = Path(self.__database,self.__collection).manage_File
        self.__manage_path_File = Path(self.__database,self.__collection).manage_path_File

        self.__Unique = "Unique"
        self.__Text = "Text"

        os.makedirs(self.__manage_path_File,exist_ok=True)
        if os.path.exists(self.__manage_File):
            pass
        else:
            FileHandler(self.__manage_File).write(EMPTY_DATABASE)


        self.manage_json_data = FileHandler(self.__manage_File).read() 


    def __inital_configration_file(self,filepath:str,writable_data:dict):
        
        if os.path.exists(filepath):
            pass
        else:
            FileHandler(filepath).write(writable_data)


    def __createFile(self,folder_path_save_independent_value:str):
        os.makedirs(folder_path_save_independent_value,exist_ok=True)
        
        

        
        filename = HadiTechnique("1").map_number_to_range()

       
        
        __db_full_path_file = os.path.join(self.__db_full_path,f"{HASH(filename).encode()}.json")
        self.__inital_configration_file(__db_full_path_file,EMPTY_FILE)

        if  self.manage_json_data.get('collection') == []:
            
            self.manage_json_data.get('collection').append(
                {self.__collection:[self.__schema]}
                )

            self.manage_json_data.get('working_file').append(
                {self.__collection:[{
                    "fileName":f"{filename}",
                    "ID":""
                }]}
                )
            
            

        elif self.manage_json_data.get('collection')[0].get(self.__collection) == None:
            self.manage_json_data.get('collection')[0].update(
                {self.__collection:[self.__schema]}
                )
           


        elif self.manage_json_data.get('collection')[0][self.__collection]:
            
            self.manage_json_data['collection'][0].get(self.__collection)[0] = self.__schema
            
          




        if self.manage_json_data.get('working_file')[0].get(self.__collection) == None:

            

            self.manage_json_data.get('working_file')[0].update(
                {self.__collection:[{
                    "fileName":f"{filename}",
                    "ID":""
                }]}
                )



        FileHandler(self.__manage_File).write(self.manage_json_data)
        return 0
     


    def validate(self):
        for key, value in self.__schema.items():
            
            if value not in SCHEMARULE:
                return {
                    "status":401,
                    "message":"Invalid Schema"
                }
            if key == 'id' or key == 'ID' or key == '__id' or key == '__ID':
                
                return {
                    "status":400,
                    "message":"ID NOT Required ( ID Auto Generated )"
                }
            elif "" == value:
            
                return {
                    "status":400,
                    "message":"Empty Value"
                }
            elif self.__Unique == value:
                


                if self.manage_json_data.get('reference_value') == []:
                
                    self.manage_json_data.get('reference_value').append(
                        {self.__collection:[{
                            key:1
                        }]}
                        )
                else:
                    
                    
                    if self.manage_json_data.get('reference_value')[0].get(self.__collection) == None:
                        
                        self.manage_json_data.get('reference_value')[0].update(
                        {self.__collection:[{
                            key:1
                        }]}
                        )
                    

                    elif self.manage_json_data.get('reference_value')[0][self.__collection][0].get(key) != None:
                        pass
                    else:
                        self.manage_json_data.get('reference_value')[0][self.__collection][0][key] = 1
                 


                self.__unique_folder_path = os.path.join(self.__db_full_path,HASH(key).encode())
                self.__createFile(self.__unique_folder_path)

                __unique_folder_path_file = os.path.join(self.__unique_folder_path,f"{HASH('1').encode()}.json")
                self.__inital_configration_file(__unique_folder_path_file,EMPTY_FILE)

            elif self.__Text == value:
                

                if self.manage_json_data.get('reference_value') == []:
                
                    self.manage_json_data.get('reference_value').append(
                        {self.__collection:[{
                            key:1
                        }]}
                        )
                else:
                    if self.manage_json_data.get('reference_value')[0].get(self.__collection) == None:
                        self.manage_json_data.get('reference_value')[0].update(
                        {self.__collection:[{
                            key:1
                        }]}
                        )
                    

                    elif self.manage_json_data.get('reference_value')[0][self.__collection][0].get(key) != None:
                        pass
                    else:
                        self.manage_json_data.get('reference_value')[0][self.__collection][0][key] = 1
                    
                        
                    


                self.__No_unique_folder_path = os.path.join(self.__db_full_path,HASH(key).encode())
                self.__createFile(self.__No_unique_folder_path)

                __No_unique_folder_path_file = os.path.join(self.__No_unique_folder_path,f"{HASH('1').encode()}.json")
                self.__inital_configration_file(__No_unique_folder_path_file,EMPTY_FILE)
                
                
            elif "Hash" == value:
                pass
            elif "Image" == value:
                pass
            
            else:
                return False
        
            

def arrangeData(schema, data):
    arranged_data = {key: data.get(key, None) for key in schema.keys()}
    return arranged_data



from HadiDB.paths import Path
from HadiDB.engine import FileHandler
from HadiDB.auth import Auth
from HadiDB.schema import arrangeData
import os
from HadiDB.write import Write
from HadiDB.haditechnique import HadiTechnique
from HadiDB.hash import HASH




EMPTY_FILE  = {
    "data": []
    }


class Insert:
    def __init__(self,
            username:str,
            password:str,
            database:str,
            collection:str,
            data:dict
                 ) -> None:
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
        self.__data = data
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path
        self.__manage_File = Path(self.__database,self.__collection).manage_File
        self.__manage_json_file_data = FileHandler(self.__manage_File).read() 

        self.filepath_unique = None
        self.unique_file_data = None
        self.filepath_text = None
        self.no_unique_file_data = None
    

 


    def __is_collection_exist(self):
        if self.__manage_json_file_data.get('collection') == []:
            return 404

        elif self.__manage_json_file_data.get('collection')[0].get(self.__collection) == None:
            return 404
        else:
            return 200


    def __write_working_file_ID_and_validate_unique_vlaues(self):
        
        if self.__manage_json_file_data['working_file'][0][self.__collection][0]['ID'] != '':
            self.__ID =  int(self.__manage_json_file_data['working_file'][0][self.__collection][0]['ID']) + 1
            self.__manage_json_file_data['working_file'][0][self.__collection][0]['ID'] = self.__ID
            
          
        else:
            
            self.__manage_json_file_data['working_file'][0][self.__collection][0]['ID'] = 1
            self.__ID = 1
            
        if self.__write_unique_values().get('status') != 409:
            
           
            return {
                "status":200,
                "filename":HASH(HadiTechnique(self.__ID).map_number_to_range()).encode()
            }
             
        else:
        
            return {
                    "status":409,
                    "error": "Conflict",
                    "message": "Data already exists"
                        }
            

    def count_unique_vlaues(self):
        self.total_keys = []
        for value in self.__schema.values():
            if value == "Unique" or value == "Text":
                self.total_keys.append(value)
        return self.total_keys


    def __write_unique_values(self):
        self.__cache_data = []
        self.__count = 0




        for key,value in self.__schema.items():
            
            self.reference_value = self.__manage_json_file_data['reference_value'][0].get(self.__collection)[0]
            
           



            if value == "Unique":
                
                self.__count = self.__count + 1
                __key = HASH(key).encode()

                

                self.filepath_unique = os.path.join(self.__db_full_path,__key,f"{HASH(str(self.reference_value[key])).encode()}.json")
                self.unique_file_data = FileHandler(self.filepath_unique).read()

               
               
                if self.unique_file_data['data'] != []:
                    if len(self.unique_file_data['data'][0]) >= 1000: # Single Chunk size write the data
                        
                        

                        self.filepath_unique = os.path.join(self.__db_full_path,__key,f"{HASH(int(self.reference_value[key]) + 1).encode()}.json")
                        FileHandler(self.filepath_unique).write(EMPTY_FILE)
                        
                        self.unique_file_data = FileHandler(self.filepath_unique).read()
                        self.reference_value[key] = int(self.reference_value[key]) + 1
                        

            
                       
                
                
                

                


                
                if self.unique_file_data['data'] == []:

                    self.unique_file_data['data'].append({
                        self.__data.get(key):self.__ID
                    })
                    
                    self.__cache_data.append({
                        
                            "filepath":self.filepath_unique,
                            "data":self.unique_file_data
                        
                    })
                    

                    
                    
                else:
                    self.__count = self.__count + 1
                    if bool(self.unique_file_data['data'][0].get(self.__data.get(key,None))) != True:
                        self.unique_file_data['data'][0][self.__data.get(key)] = self.__ID
                        
                        self.__cache_data.append({
                        
                            "filepath":self.filepath_unique,
                            "data":self.unique_file_data
                        
                    })
                       

                    else:
                        
                        return {
                            "status":409,
                            "error": "Conflict",
                            "message": "Data already exists"
                        }
                

            elif value == "Text":
                self.__count = self.__count + 1
                __key = HASH(key).encode()
                self.filepath_text = os.path.join(self.__db_full_path,__key,f"{HASH(int(self.reference_value[key])).encode()}.json")
                self.no_unique_file_data = FileHandler(self.filepath_text).read()



                if self.no_unique_file_data['data'] != []:
                    if len(self.no_unique_file_data['data']) >= 1000:
                        
                        

                        self.filepath_text = os.path.join(self.__db_full_path,__key,f"{HASH(int(self.reference_value[key]) + 1).encode()}.json")
                        FileHandler(self.filepath_text).write(EMPTY_FILE)
                        self.no_unique_file_data = FileHandler(self.filepath_text).read()
                        
                        self.reference_value[key] = int(self.reference_value[key]) + 1
                        
                        




                if self.no_unique_file_data['data'] == []:
                    self.no_unique_file_data['data'].append(
                       
                        [self.__data.get(key),self.__ID]
                        
                    )
                    
                    self.__cache_data.append({
                        
                            "filepath":self.filepath_text,
                            "data":self.no_unique_file_data
                        
                    })
                  
                else:
                    self.__count = self.__count + 1
                    self.no_unique_file_data['data'].append(
                        
                        [self.__data.get(key),self.__ID]
                    )
                    
                    self.__cache_data.append({
                        
                            "filepath":self.filepath_text,
                            "data":self.no_unique_file_data
                        
                    })
                    

            
        if len(self.count_unique_vlaues()) == len(self.__cache_data):
            
            for i in self.__cache_data:
                FileHandler(i['filepath']).write(i['data'])
        else:
            return {"status":404}




        
        return {
            "status":200
        }
                    
                


    def insert(self):
        if Auth(self.__username,self.__password)._Authenticate() == None:
            return {
                "status":401,
                "message":"Wrong Username Password"
            }

        if self.__is_collection_exist() == 200:
            
            self.__schema = self.__manage_json_file_data.get('collection')[0].get(self.__collection)[0]

           

            
            validate_schema = self.__data.keys() == self.__schema.keys()

            if validate_schema == True:
                
                arrange_data = (arrangeData(self.__schema,self.__data))
                
                filename = self.__write_working_file_ID_and_validate_unique_vlaues()

                if filename['status'] == 409:
                    return {
                            "status":409,
                            "error": "Conflict",
                            "message": "Data already exists"
                        }


                else:
                    arrange_data['id'] = self.__ID
                    

                    filepath = os.path.join(self.__db_full_path,f"{filename['filename']}.json")
                    if os.path.exists(filepath) == False:
                        FileHandler(os.path.join(self.__db_full_path,f"{filename['filename']}.json")).write(
                            {
                                "data":[]
                            }
                        )
                    


                    readFile = FileHandler(os.path.join(self.__db_full_path,f"{filename['filename']}.json"))
                    
                    read_file = readFile.read()

                

                    if read_file['data'] == []:
                        read_file['data'].append({
                            arrange_data.get('id'):arrange_data
                        })


                    else:
                        
                        if len(read_file['data'][0]) >= 1000: # single Chunk size write the data
                            self.__manage_json_file_data.get('working_file')[0].get(self.__collection)[0]['fileName'] = HASH(HadiTechnique(self.__ID).map_number_to_range()).encode()
                            
                            


                        read_file['data'][0].update({
                        arrange_data.get('id'): arrange_data
                        
                    } )

                    

                    FileHandler(self.__manage_File).write(self.__manage_json_file_data)
                    readFile.write(read_file)
                    return {
                        "status":200,
                        "message":"Data insert successfully",
                        "data":arrange_data
                    }
                
              

                
            else:
                return {
                    "status":404,
                    "message":"Invalid Schema missmatch key"
                }

        
            

        else:
            return {
                "status":404,
                "message":"collection not found"
            }


import io
import json
import os
import gzip

class FileHandler1:
    def __init__(self, filepath:str, encoding='utf-8'):
        self.filepath = filepath
        self.encoding = encoding
        self._object_read = None
        self._object_write = None

    def write(self,content:dict):
        
        temp_file_path = self.filepath + "_temp.json"
        
        self._object_write = io.open(temp_file_path, 'w', encoding=self.encoding)
        json.dump(content, self._object_write, indent=4)
        self._object_write.close()
        os.replace(temp_file_path, self.filepath)

        return None


    def read(self):

        self._object_read = io.open(self.filepath, 'r', encoding=self.encoding)
        data =  json.load(self._object_read)
        self._object_read.close()
        return data




class FileHandler:
    def __init__(self, filepath: str, encoding='utf-8'):
        self.filepath = filepath
        self.encoding = encoding
        self._object_read = None
        self._object_write = None

    def write(self, content: dict):
        try:
            temp_file_path = self.filepath + "_temp.json"
        
            with gzip.open(temp_file_path, 'wt', encoding=self.encoding) as self._object_write:
                json.dump(content, self._object_write, indent=4)
            os.replace(temp_file_path, self.filepath)
        except:
            return {
                "status":404
            }
            # self._object_write.close()

    def read(self):
        
        try:

            with gzip.open(self.filepath, 'rt', encoding=self.encoding) as self._object_read:
                data = json.load(self._object_read)
            # self._object_read.close()
            return data
        except:
            return {
                "status":404
            }
    






from HadiDB.insert import Insert
from HadiDB.createDB import Database
from HadiDB.getbyID import GetByID
from HadiDB.delete import Delete
from HadiDB.auth import Auth
from HadiDB.update import Update
from HadiDB.config import Config
from HadiDB.get_all_data import GETALLDATA
from HadiDB.getbyKey import GetByKey
from HadiDB.count import Count
from HadiDB.countbyAll import CountByAll
from HadiDB.getbyKeys import GetByKeys
from HadiDB.delete_database import DeleteDatabase
from HadiDB.delete_collection import DeleteCollection
from filelock import FileLock



class User:
    def __init__(self,username,password) -> None:
        self.__username = username
        self.__password = password
        self.lock = FileLock("{}.lock")
    def createUser(self):
        with self.lock:
            return Auth(self.__username,self.__password)._createUser()
            

    def authentication(self):
        return Auth(self.__username,self.__password)._Authenticate()



class Configuration:
    def __init__(self,__database:str=None,__collection:str=None) -> None:
        self.__database = __database
        self.__collection = __collection
    def get_collection(self):
        return Config(self.__database).getallcollection()
    def get_database(self):
        return Config().getalldatabase()

    def get_schema(self):
        return Config(self.__database,self.__collection).getschema()



class DatabaseDeletionService:
    def __init__(self,username,password,database=None,collection=None) -> None:
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
    

    def authentication(self):
        return Auth(self.__username,self.__password)._Authenticate()


    def deleteDatabase(self):
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return DeleteDatabase(self.__database).delete()

    def deleteCollection(self):
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return DeleteCollection(self.__database,self.__collection).delete()

    



class Operation:
    def __init__(self,
            username:str,
            password:str,
            database:str,
            collection:str,
                 ) -> None:
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection
        self.lock = FileLock("{}.lock")

    def authentication(self):
        return Auth(self.__username,self.__password)._Authenticate()


    def create_database(self,schema:dict=None):
        
        if type(schema).__name__ != "dict":
            return {
                "status":400,
                "message":"pass only dict"
            }
        
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        with self.lock:
            return Database(self.__username,self.__password,self.__database,self.__collection,schema).create()


    def insert(self,data:dict=None):

        if type(data).__name__ != "dict":
            return {
                "status":400,
                "message":"pass only dict"
            }

        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        with self.lock:
            
            return Insert(self.__username,self.__password,self.__database,self.__collection,data).insert()

    def getbyID(self,objectID:int=None):

        if type(objectID).__name__ != "int":
            return {
                "status":400,
                "message":"pass only int"
            }
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return GetByID(self.__username,self.__password,self.__database,self.__collection).get(objectID)

    def update(self,objectID:int=None,data:dict=None):

        
        if type(objectID).__name__ != "int":
            return {
                "status":400,
                "message":"Missing ID"
            }



        if type(data).__name__ != "dict":
            return {
                "status":400,
                "message":"missing dict or pass only wrong data"
            }


        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        with self.lock:
            return Update(self.__username,self.__password,self.__database,self.__collection,objectID,data).update()

    def delete(self,objectID:int=None):

        if type(objectID).__name__ != "int":
            return {
                "status":400,
                "message":"pass only int"
            }


        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        with self.lock:
            return Delete(self.__username,self.__password,self.__database,self.__collection,objectID).delete()

 
    def getAll(self):
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return GETALLDATA(self.__database,self.__collection).getall()


    def getbykey(self,data:dict=None):

        if type(data).__name__ != "dict" or data == {}:
            return {
                "status":400,
                "message":"pass only dict"
            }


        if len(data.items()) > 1:
            return {
                "message":"getbykey Support only single key Multiple keys use getbykeys"
            }

        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return GetByKey(self.__username,self.__password,self.__database,self.__collection,data).get()


    def getbykeyCount(self,data:dict=None):


        if type(data).__name__ != "dict" or data == {}:
            return {
                "status":400,
                "message":"pass only dict"
            }

        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return GetByKey(self.__username,self.__password,self.__database,self.__collection,data).count()



    def count(self):
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return Count(self.__database,self.__collection).get()


    def countbyAll(self):
        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return CountByAll(self.__database,self.__collection).get()
        
    def getbykeys(self,data:dict=None):

        
        if type(data).__name__ != "dict" or data == {}:
            return {
                "status":400,
                "message":"pass only dict"
            }


        if self.authentication() == None:
            return {
                "status":404,
                "message":"wrong username and password"
            }
        return GetByKeys(self.__database,self.__collection).get(data)



 




from HadiDB.paths import Path
from HadiDB.engine import FileHandler
import os
import itertools


class GETALLDATA:
    def __init__(self,database:str,collection:str):
        self.__database = database
        self.__collection = collection
        self.db_full_path = Path(self.__database,self.__collection).db_full_path

    def getall(self):
        try:
            all_data = []
            
            listdir =  [file for file in os.listdir(self.db_full_path) if file.endswith('.json')]
            
            
            for file in itertools.islice(listdir, len(listdir)):
                
            
                result = FileHandler(
                    os.path.join(self.db_full_path,file)
                ).read()

                data = list(result['data'][0].values())

                all_data.append(data)


            return all_data
        except Exception as e:
            
            return {
                "status":404,
                "message":"DB is Empty"
            }






from HadiDB.paths import Path
from HadiDB.engine import FileHandler
from HadiDB.auth import Auth
from HadiDB.hash import HASH
from HadiDB.haditechnique import HadiTechnique
import os
import itertools


class Delete:
    def __init__(self,
                username:str,
                password:str,
                database:str,
                collection:str,
                objectid:int,
                 ) -> None:
        self.__id = objectid
        self.__username = username
        self.__password = password
        self.__database = database
        self.__collection = collection

        
        self.__db_full_path = Path(self.__database,self.__collection).db_full_path
        self.__manage_File = Path(self.__database,self.__collection).manage_File
        self.__manage_json_file_data = FileHandler(self.__manage_File).read() 
        self.__delete_count = self.__manage_json_file_data.get('delete_count')



    def __is_collection_exist(self):
        if self.__manage_json_file_data.get('collection') == []:
            return 404
        elif self.__manage_json_file_data.get('collection')[0].get(self.__collection) == None:
            return 404
        else:
            return 200



    def delete(self):
        if Auth(self.__username,self.__password)._Authenticate() == None:
            return {
                "status":401,
                "message":"Wrong Username Password"
            }

        if self.__is_collection_exist() == 200:
            self.__schema = self.__manage_json_file_data.get('collection')[0].get(self.__collection)[0]
            fileName = HadiTechnique(self.__id).map_number_to_range()
            
            
            self.actual_data_file_path = os.path.join(self.__db_full_path,f"{HASH(f'{fileName}').encode()}.json")
            try:
                self.get_actual_data = FileHandler(self.actual_data_file_path).read()
            except:
                return {
                    "status":404,
                    "message":"ID not Found"
                }

               
            if self.get_actual_data['data'][0].get(str(self.__id)) == None:
                return {
                    "status":404,
                    "message":"ID not Found"
                }

            if self.get_actual_data['data'] == []:
                return {
                    "status":404,
                    "message":"collection is Empty"
                }
            
            if self.__delete_unique_values()['status'] == 200:

                if str(self.__id) in self.get_actual_data['data'][0]:
                    del (self.get_actual_data['data'][0][f"{self.__id}"])
                    FileHandler(self.actual_data_file_path).write(self.get_actual_data)

                    if self.__manage_json_file_data.get('delete_count') == []:

                        self.__manage_json_file_data.get('delete_count').append({
                            self.__collection:1
                        })
                    else:
                        
                        if self.__manage_json_file_data.get('delete_count')[0].get(self.__collection) != None:
                            self.__manage_json_file_data.get('delete_count')[0][self.__collection] += 1
                        else:
                            self.__manage_json_file_data.get('delete_count').append({
                            self.__collection:1
                        })
                    

                    FileHandler(self.__manage_File).write(self.__manage_json_file_data) 

                    return {
                        "status":200,
                        "message":"data delete successful"
                    }
        else:
            return {
                        "status":404,
                        "message":"collection not found"
                    }


    def __delete_unique_values(self):
        self.__cache_data = []
        self.__count = 0
        for key,value in self.__schema.items():
            if value == "Unique":
                self.__count = self.__count + 1
                __key = HASH(key).encode()


                self.folder_path = os.path.join(self.__db_full_path,__key)
                # files = os.listdir(self.folder_path)
                files = [file[:-5] for file in os.listdir(self.folder_path) if file.endswith('.json')]
                get_actual_key_value = self.get_actual_data['data'][0].get(str(self.__id))[key]
                
                for filename in itertools.islice(files,len(files)):
                    
                    # self.filepath_unique = os.path.join(self.__db_full_path,__key,f"{HASH(filename).encode()}.json")
                    self.filepath_unique = os.path.join(self.__db_full_path,__key,f"{filename}.json")
                    self.unique_file_data = FileHandler(self.filepath_unique).read()


                    try:
                        
                        # get_actual_key_value = self.get_actual_data['data'][0].get(str(self.__id))[key]
                        
                        del self.unique_file_data['data'][0][get_actual_key_value]
                        self.__cache_data.append({
                                
                                    "filepath":self.filepath_unique,
                                    "data":self.unique_file_data
                                
                            })
                    except:
                        pass


            elif value == "Text":
                self.__count = self.__count + 1
                __key = HASH(key).encode()

                self.folder_path = os.path.join(self.__db_full_path,__key)
                # files = os.listdir(self.folder_path)
                files = [file[:-5] for file in os.listdir(self.folder_path) if file.endswith('.json')]
                get_actual_key_value = self.get_actual_data['data'][0].get(str(self.__id))[key]
                
                for filename in itertools.islice(files,len(files)):
                    
                    # self.filepath_text = os.path.join(self.__db_full_path,__key,f"{HASH(filename).encode()}.json")
                    self.filepath_text = os.path.join(self.__db_full_path,__key,f"{filename}.json")
                    self.no_unique_file_data = FileHandler(self.filepath_text).read()
                
                    for item in itertools.islice(self.no_unique_file_data['data'] ,len(self.no_unique_file_data['data'])):
                        if item[0] == get_actual_key_value:
                            index = self.no_unique_file_data['data'].index(item)
                            
                            self.no_unique_file_data['data'].pop(int(index))
                            
                            self.__cache_data.append({
                                    
                                        "filepath":self.filepath_text,
                                        "data":self.no_unique_file_data
                                    
                                })
                        else:
                            pass



                #
        
        for i in self.__cache_data:
            FileHandler(i['filepath']).write(i['data'])

        


        return {
            "status":200
        }


from setuptools import setup, find_packages



VERSION = '0.1'
DESCRIPTION = "HadiDB is a lightweight, extremely horizontally scalable database written in Python."


# Setting up
setup(
    name="HadiDB",
    version=VERSION,
    author="mominiqbal1234",
    author_email="<mominiqbal1214@gmail.com>",
    description=DESCRIPTION,
    long_description="""
# HadiDB
HadiDB is a lightweight, extremely horizontally scalable database written in Python

# How to install hadidb

```python
pip install hadidb
```

## Create User HadiDB
Creates a new user with the `example` username  `admin` and `example` password `admin` using `createUser()`. It then authenticates the same user by calling the `authentication()` method. 

```python
from HadiDB.operation import User
user = User("admin", "admin")
user.createUser() # Creating a new user in the HadiDB
user.authentication() # Authenticating the HadiDB user
```
##### Result:

```josn
{'status': 200, 'message': 'Database user Created'}
```


## Create Databse , Collection and Schema
This code sets up user credentials and a schema for a database collection. It initializes a database operation using the `Operation` class with the specified `username`, `password`, `database`, and `collection`. Finally, it inserts the provided `data` into the collection and stores the result.

```python
from HadiDB.operation import Operation

username = "admin"
password = "admin"
database = "mefiz.com"
collection = "authUser"

schema = {
    "username":"Unique",
    "password":"Hash",
    "cnic":"Unique",
    "picture":"Image",
    "bio":"Text"
}
db = Operation(username,password,database,collection)
db.create_database(schema)
```
## Insert Data
Insert Data into the Collection use `db.insert(data)` inserts the `data` into the specified database collection.
```python
from HadiDB.operation import Operation

username = "admin"
password = "admin"
database = "mefiz.com"
collection = "authUser"


db = Operation(username,password,database,collection)

data = {
    "username":"hadidb",
    "password":"12345",
    "cnic":"123232442",
    "picture":"user/my/hadidb.jpg",
    "bio":"HadiDB is the Best ;)"
}


result = db.insert(data)
print(result)
```
##### Result:
```json
{
'status': 200, 
'message': 'Data insert successfully',
'data': {
    'username': 'hadidb', 
    'password': '12345', 
    'cnic': '123232442', 
    'picture': 'user/my/hadidb.jpg', 
    'bio': 'HadiDB is the Best ;)',
     'id': 1
     }
}


```

## Update Data
Update Data `db.update(1, update_data)` updates the record with the ID `1` in the database using the provided `update_data`.
```python
from HadiDB.operation import Operation

username = "admin"
password = "admin"
database = "mefiz.com"
collection = "authUser"


db = Operation(username,password,database,collection)


update_data = {     
    "username": "hadidb_update",
    "password": "123455",
    "cnic": "1232324423",
    "picture": "user/my/hadidb1.jpg",
    "bio": "HadiDB is the Best ;) update bio" 
}

result = db.update(1,update_data)
print(result)
```
##### Result:
```json
{
    'status': 200, 
    'message': 'Data Update successfully',
    'data': {
    'username': 'hadidb_update', 
    'password': '123455', 
    'cnic': '1232324423', 
    'picture': 'user/my/hadidb1.jpg', 
    'bio': 'HadiDB is the Best ;) update bio', 
    'id': 1
    }
}
```


## GetByID
The unique identifier (ID) of the document you want to retrieve specific object or an error if the document does not exist.

```python
result = db.getbyID(1)
print(result)
```

## Get All Object
The `getAll` method retrieves all documents from the specified collection in the database.
```python
result = db.getAll()
print(result)
```

## GetByKey
The `getbykey` method retrieves all documents from the database where the specified key-value pair matches. `Not Support multi keys values pairs`
```python
result = db.getbykey({
    "username":"momin"
 })
print(result)
```

## GetByKeys

The getbykeys function uses an implicit `AND (&&)`operation. Both conditions `Example (cnic and bio)` if matched key values in the database then return the matched object.

```python
result = db.getbykeys({
    "cnic":"123232442",
    "bio":"HadiDB is the Best ;) update bio"
})
print(result)
```


## Count 
The `count` method returns the total number of documents (or objects) present in the specified collection in the database.

```python
result = db.count()
print(result)
```
##### Result:
```json
{'status': 200, 'count': 1}
```

## GeyByKeyCount
The `getbykeyCount` method counts the number of documents in the collection where the specified key-value pair matches.
```python
result = db.getbykeyCount({
    "username":"momin"
    })
```




## Delete 
Deletes a document from the database by its unique identifier (`id`)

```python
result = db.delete(1)
print(result)
```
##### Reuslt:
```json
{'status': 200, 'message': 'data delete successful'}
```


## Get All Database 
Retrieves all available databases by using the `get_database()` method of the `Configuration` class
```python
from HadiDB.operation import Configuration

print(Configuration().get_database())
```
## Get All Collection
Retrieves all collections from a specific database using the `get_collection() ` method of the `Configuration` class.

```python
from HadiDB.operation import Configuration

database = "mefiz.com"
print(Configuration(database).get_collection())
```
## Get Schema of Specfic Collection
Return Schema of a specific collection by using ` get_schema() `method from the `Configuration` class.
```python
from HadiDB.operation import Configuration
database = "mefiz.com"
collection = "authUser"
print(Configuration(database,collection).get_schema())
```



## Delete Collection
Deletes a specific collection from a database using the `deleteCollection()` method of the `DatabaseDeletionService` class.
```python
from HadiDB.operation import DatabaseDeletionService

db = DatabaseDeletionService(username,password,database,collection)
print(db.deleteCollection())
```
## Delete Database 
Deletes Database using the `deleteDatabase()` method of the `DatabaseDeletionService` class.
```python
from HadiDB.operation import DatabaseDeletionService

db = DatabaseDeletionService(username,password,database,collection)
print(db.deleteDatabase())
```

##### GitHub : https://github.com/MominIqbal-1234/hadidb
##### Check Our Site : https://mefiz.com/about </br>
##### Developed by : Momin Iqbal

    """,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=["filelock"],
    keywords=['python','pyjsondb''python database', 'hadidb', 'python hadidb','hadidb django'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from HadiDB.paths import Path
from HadiDB.engine import FileHandler
import itertools
import os

class Count:
    def __init__(self,database:str,collection:str) -> None:

        self.__database = database
        self.__collection = collection
        self.db_full_path = Path(self.__database,self.__collection).db_full_path
        
    def get(self):
        try:
            length__all_data = []
            
            listdir =  [file for file in os.listdir(self.db_full_path) if file.endswith('.json')]
            
            # for file in listdir:
            for file in itertools.islice(listdir, len(listdir)):
                
            
                result = FileHandler(
                    os.path.join(self.db_full_path,file)
                ).read()

                length_data = len(result['data'][0])
                
                length__all_data.append(length_data)
                


            
            return {
                "status":200,
                "count":sum(length__all_data)
            }

        except Exception as e:
            
            return {
                "status":404,
                "message":"DB is Empty"
            }


from HadiDB.hash import HASH
from HadiDB.engine import FileHandler
from HadiDB.paths import Path
import os




EMPTY_DATABASE  = {
    "database": [],
    "collection": [],
    "schema": [],
    "working_file": [],
    "uniqueValues": [],
    "delete_count": []
    }

class Auth:
    def __init__(self,username:str , password: str) -> None:
        self.username = HASH(username).encode()
        self.password = HASH(password).encode()
        # ------------------Files --------------------------
    
       
        self.bin_path = Path().bin_full_path
        self.auth_file = Path().auth_File
        self.manage_File_bin = Path().manage_File_bin


    def _createUser(self):
        os.makedirs(self.bin_path, exist_ok=True)


        """========================== Create Auth.json ==========================="""
        FileHandler(self.auth_file).write({
            "username":self.username,
            "password":self.password
        })
        """========================== END Auth.json ==========================="""
        
        """========================== Create Manage.json ==========================="""
        if os.path.exists(self.manage_File_bin):
            pass
        else:
            FileHandler(self.manage_File_bin).write(EMPTY_DATABASE)
            return {
                "status":200,
                "message":"Database user Created"
            }
        """========================== End Manage.json ==========================="""
    def _Authenticate(self):
        user = FileHandler(self.auth_file).read()
        if user.get('username') == self.username and user.get('password') == self.password:
            return {
                "status":200,
                "message":"login successful"
            }
        else:
            return None  


